from collections.abc import Mapping
from typing import Any, TypeVar, Optional, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast
from typing import cast, Union
from typing import Union

if TYPE_CHECKING:
  from ..models.slim_mini_dto import SlimMiniDto
  from ..models.user_dto import UserDto





T = TypeVar("T", bound="GroupDto")



@_attrs_define
class GroupDto:
    """ 
        Attributes:
            id (Union[Unset, str]):
            name (Union[Unset, str]):
            description (Union[None, Unset, str]):
            workspace_id (Union[Unset, str]):
            users (Union[Unset, list['UserDto']]):
            minis (Union[Unset, list['SlimMiniDto']]):
     """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    description: Union[None, Unset, str] = UNSET
    workspace_id: Union[Unset, str] = UNSET
    users: Union[Unset, list['UserDto']] = UNSET
    minis: Union[Unset, list['SlimMiniDto']] = UNSET





    def to_dict(self) -> dict[str, Any]:
        from ..models.slim_mini_dto import SlimMiniDto
        from ..models.user_dto import UserDto
        id = self.id

        name = self.name

        description: Union[None, Unset, str]
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        workspace_id = self.workspace_id

        users: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.users, Unset):
            users = []
            for users_item_data in self.users:
                users_item = users_item_data.to_dict()
                users.append(users_item)



        minis: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.minis, Unset):
            minis = []
            for minis_item_data in self.minis:
                minis_item = minis_item_data.to_dict()
                minis.append(minis_item)




        field_dict: dict[str, Any] = {}

        field_dict.update({
        })
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if workspace_id is not UNSET:
            field_dict["workspaceId"] = workspace_id
        if users is not UNSET:
            field_dict["users"] = users
        if minis is not UNSET:
            field_dict["minis"] = minis

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.slim_mini_dto import SlimMiniDto
        from ..models.user_dto import UserDto
        d = dict(src_dict)
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        def _parse_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        description = _parse_description(d.pop("description", UNSET))


        workspace_id = d.pop("workspaceId", UNSET)

        users = []
        _users = d.pop("users", UNSET)
        for users_item_data in (_users or []):
            users_item = UserDto.from_dict(users_item_data)



            users.append(users_item)


        minis = []
        _minis = d.pop("minis", UNSET)
        for minis_item_data in (_minis or []):
            minis_item = SlimMiniDto.from_dict(minis_item_data)



            minis.append(minis_item)


        group_dto = cls(
            id=id,
            name=name,
            description=description,
            workspace_id=workspace_id,
            users=users,
            minis=minis,
        )

        return group_dto




from collections.abc import Mapping
from typing import Any, TypeVar, Optional, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field
import json
from .. import types

from ..types import UNSET, Unset

from ..types import File, FileTypes
from ..types import UNSET, Unset
from io import BytesIO
from typing import cast
from typing import cast, Union
from typing import Union






T = TypeVar("T", bound="AddAttachmentsBody")



@_attrs_define
class AddAttachmentsBody:
    """ 
        Attributes:
            files (Union[None, Unset, list[File]]):
     """

    files: Union[None, Unset, list[File]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        files: Union[None, Unset, list[FileTypes]]
        if isinstance(self.files, Unset):
            files = UNSET
        elif isinstance(self.files, list):
            files = []
            for files_type_0_item_data in self.files:
                files_type_0_item = files_type_0_item_data.to_tuple()

                files.append(files_type_0_item)


        else:
            files = self.files


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if files is not UNSET:
            field_dict["files"] = files

        return field_dict


    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.files, Unset):
            if isinstance(self.files, list):

                for files_type_0_item_element in self.files:
                    files.append(("files", files_type_0_item_element.to_tuple()))
            else:
                files.append(("files", (None, str(self.files).encode(), "text/plain")))



        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))



        return files


    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        def _parse_files(data: object) -> Union[None, Unset, list[File]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                files_type_0 = []
                _files_type_0 = data
                for files_type_0_item_data in (_files_type_0):
                    files_type_0_item = File(
                         payload = BytesIO(files_type_0_item_data)
                    )



                    files_type_0.append(files_type_0_item)

                return files_type_0
            except: # noqa: E722
                pass
            return cast(Union[None, Unset, list[File]], data)

        files = _parse_files(d.pop("files", UNSET))


        add_attachments_body = cls(
            files=files,
        )


        add_attachments_body.additional_properties = d
        return add_attachments_body

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, Optional, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast
from typing import Union

if TYPE_CHECKING:
  from ..models.record_dto import RecordDto





T = TypeVar("T", bound="PaginatedListOfRecordDto")



@_attrs_define
class PaginatedListOfRecordDto:
    """ 
        Attributes:
            items (Union[Unset, list['RecordDto']]):
            page_number (Union[Unset, int]):
            total_pages (Union[Unset, int]):
            total_count (Union[Unset, int]):
            has_previous_page (Union[Unset, bool]):
            has_next_page (Union[Unset, bool]):
     """

    items: Union[Unset, list['RecordDto']] = UNSET
    page_number: Union[Unset, int] = UNSET
    total_pages: Union[Unset, int] = UNSET
    total_count: Union[Unset, int] = UNSET
    has_previous_page: Union[Unset, bool] = UNSET
    has_next_page: Union[Unset, bool] = UNSET





    def to_dict(self) -> dict[str, Any]:
        from ..models.record_dto import RecordDto
        items: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.items, Unset):
            items = []
            for items_item_data in self.items:
                items_item = items_item_data.to_dict()
                items.append(items_item)



        page_number = self.page_number

        total_pages = self.total_pages

        total_count = self.total_count

        has_previous_page = self.has_previous_page

        has_next_page = self.has_next_page


        field_dict: dict[str, Any] = {}

        field_dict.update({
        })
        if items is not UNSET:
            field_dict["items"] = items
        if page_number is not UNSET:
            field_dict["pageNumber"] = page_number
        if total_pages is not UNSET:
            field_dict["totalPages"] = total_pages
        if total_count is not UNSET:
            field_dict["totalCount"] = total_count
        if has_previous_page is not UNSET:
            field_dict["hasPreviousPage"] = has_previous_page
        if has_next_page is not UNSET:
            field_dict["hasNextPage"] = has_next_page

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.record_dto import RecordDto
        d = dict(src_dict)
        items = []
        _items = d.pop("items", UNSET)
        for items_item_data in (_items or []):
            items_item = RecordDto.from_dict(items_item_data)



            items.append(items_item)


        page_number = d.pop("pageNumber", UNSET)

        total_pages = d.pop("totalPages", UNSET)

        total_count = d.pop("totalCount", UNSET)

        has_previous_page = d.pop("hasPreviousPage", UNSET)

        has_next_page = d.pop("hasNextPage", UNSET)

        paginated_list_of_record_dto = cls(
            items=items,
            page_number=page_number,
            total_pages=total_pages,
            total_count=total_count,
            has_previous_page=has_previous_page,
            has_next_page=has_next_page,
        )

        return paginated_list_of_record_dto




from collections.abc import Mapping
from typing import Any, TypeVar, Optional, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast, Union
from typing import Union






T = TypeVar("T", bound="PatchMiniCommand")



@_attrs_define
class PatchMiniCommand:
    """ 
        Attributes:
            id (Union[Unset, int]):
            name (Union[None, Unset, str]):
            description (Union[None, Unset, str]):
     """

    id: Union[Unset, int] = UNSET
    name: Union[None, Unset, str] = UNSET
    description: Union[None, Unset, str] = UNSET





    def to_dict(self) -> dict[str, Any]:
        id = self.id

        name: Union[None, Unset, str]
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        description: Union[None, Unset, str]
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description


        field_dict: dict[str, Any] = {}

        field_dict.update({
        })
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id", UNSET)

        def _parse_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        name = _parse_name(d.pop("name", UNSET))


        def _parse_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        description = _parse_description(d.pop("description", UNSET))


        patch_mini_command = cls(
            id=id,
            name=name,
            description=description,
        )

        return patch_mini_command




from collections.abc import Mapping
from typing import Any, TypeVar, Optional, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
from typing import cast, Union
from typing import Union
import datetime






T = TypeVar("T", bound="UserDto")



@_attrs_define
class UserDto:
    """ 
        Attributes:
            id (Union[Unset, str]):
            email (Union[Unset, str]):
            first_name (Union[None, Unset, str]):
            last_name (Union[None, Unset, str]):
            profile_picture_url (Union[None, Unset, str]):
            created_at (Union[None, Unset, datetime.datetime]):
            updated_at (Union[None, Unset, datetime.datetime]):
            organization_ids (Union[Unset, list[str]]):
     """

    id: Union[Unset, str] = UNSET
    email: Union[Unset, str] = UNSET
    first_name: Union[None, Unset, str] = UNSET
    last_name: Union[None, Unset, str] = UNSET
    profile_picture_url: Union[None, Unset, str] = UNSET
    created_at: Union[None, Unset, datetime.datetime] = UNSET
    updated_at: Union[None, Unset, datetime.datetime] = UNSET
    organization_ids: Union[Unset, list[str]] = UNSET





    def to_dict(self) -> dict[str, Any]:
        id = self.id

        email = self.email

        first_name: Union[None, Unset, str]
        if isinstance(self.first_name, Unset):
            first_name = UNSET
        else:
            first_name = self.first_name

        last_name: Union[None, Unset, str]
        if isinstance(self.last_name, Unset):
            last_name = UNSET
        else:
            last_name = self.last_name

        profile_picture_url: Union[None, Unset, str]
        if isinstance(self.profile_picture_url, Unset):
            profile_picture_url = UNSET
        else:
            profile_picture_url = self.profile_picture_url

        created_at: Union[None, Unset, str]
        if isinstance(self.created_at, Unset):
            created_at = UNSET
        elif isinstance(self.created_at, datetime.datetime):
            created_at = self.created_at.isoformat()
        else:
            created_at = self.created_at

        updated_at: Union[None, Unset, str]
        if isinstance(self.updated_at, Unset):
            updated_at = UNSET
        elif isinstance(self.updated_at, datetime.datetime):
            updated_at = self.updated_at.isoformat()
        else:
            updated_at = self.updated_at

        organization_ids: Union[Unset, list[str]] = UNSET
        if not isinstance(self.organization_ids, Unset):
            organization_ids = self.organization_ids




        field_dict: dict[str, Any] = {}

        field_dict.update({
        })
        if id is not UNSET:
            field_dict["id"] = id
        if email is not UNSET:
            field_dict["email"] = email
        if first_name is not UNSET:
            field_dict["firstName"] = first_name
        if last_name is not UNSET:
            field_dict["lastName"] = last_name
        if profile_picture_url is not UNSET:
            field_dict["profilePictureUrl"] = profile_picture_url
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at
        if organization_ids is not UNSET:
            field_dict["organizationIds"] = organization_ids

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id", UNSET)

        email = d.pop("email", UNSET)

        def _parse_first_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        first_name = _parse_first_name(d.pop("firstName", UNSET))


        def _parse_last_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        last_name = _parse_last_name(d.pop("lastName", UNSET))


        def _parse_profile_picture_url(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        profile_picture_url = _parse_profile_picture_url(d.pop("profilePictureUrl", UNSET))


        def _parse_created_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                created_at_type_0 = isoparse(data)



                return created_at_type_0
            except: # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        created_at = _parse_created_at(d.pop("createdAt", UNSET))


        def _parse_updated_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                updated_at_type_0 = isoparse(data)



                return updated_at_type_0
            except: # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        updated_at = _parse_updated_at(d.pop("updatedAt", UNSET))


        organization_ids = cast(list[str], d.pop("organizationIds", UNSET))


        user_dto = cls(
            id=id,
            email=email,
            first_name=first_name,
            last_name=last_name,
            profile_picture_url=profile_picture_url,
            created_at=created_at,
            updated_at=updated_at,
            organization_ids=organization_ids,
        )

        return user_dto




from http import HTTPStatus
from typing import Any, Optional, Union, cast

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.paginated_list_of_record_dto import PaginatedListOfRecordDto
from ...types import UNSET, Unset
from typing import cast
from typing import cast, Union
from typing import Union



def _get_kwargs(
    *,
    body: list[str],
    mini_id: str,
    session_id: Union[None, Unset, str] = UNSET,
    page_number: Union[Unset, int] = 1,
    page_size: Union[Unset, int] = 10,

) -> dict[str, Any]:
    headers: dict[str, Any] = {}


    

    params: dict[str, Any] = {}

    params["miniId"] = mini_id

    json_session_id: Union[None, Unset, str]
    if isinstance(session_id, Unset):
        json_session_id = UNSET
    else:
        json_session_id = session_id
    params["sessionId"] = json_session_id

    params["pageNumber"] = page_number

    params["pageSize"] = page_size


    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}


    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/api/Records/external",
        "params": params,
    }

    _kwargs["json"] = body




    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs



def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[PaginatedListOfRecordDto]:
    if response.status_code == 200:
        response_200 = PaginatedListOfRecordDto.from_dict(response.json())



        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[PaginatedListOfRecordDto]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: list[str],
    mini_id: str,
    session_id: Union[None, Unset, str] = UNSET,
    page_number: Union[Unset, int] = 1,
    page_size: Union[Unset, int] = 10,

) -> Response[PaginatedListOfRecordDto]:
    """ 
    Args:
        mini_id (str):
        session_id (Union[None, Unset, str]):
        page_number (Union[Unset, int]):  Default: 1.
        page_size (Union[Unset, int]):  Default: 10.
        body (list[str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PaginatedListOfRecordDto]
     """


    kwargs = _get_kwargs(
        body=body,
mini_id=mini_id,
session_id=session_id,
page_number=page_number,
page_size=page_size,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    body: list[str],
    mini_id: str,
    session_id: Union[None, Unset, str] = UNSET,
    page_number: Union[Unset, int] = 1,
    page_size: Union[Unset, int] = 10,

) -> Optional[PaginatedListOfRecordDto]:
    """ 
    Args:
        mini_id (str):
        session_id (Union[None, Unset, str]):
        page_number (Union[Unset, int]):  Default: 1.
        page_size (Union[Unset, int]):  Default: 10.
        body (list[str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PaginatedListOfRecordDto
     """


    return sync_detailed(
        client=client,
body=body,
mini_id=mini_id,
session_id=session_id,
page_number=page_number,
page_size=page_size,

    ).parsed

async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: list[str],
    mini_id: str,
    session_id: Union[None, Unset, str] = UNSET,
    page_number: Union[Unset, int] = 1,
    page_size: Union[Unset, int] = 10,

) -> Response[PaginatedListOfRecordDto]:
    """ 
    Args:
        mini_id (str):
        session_id (Union[None, Unset, str]):
        page_number (Union[Unset, int]):  Default: 1.
        page_size (Union[Unset, int]):  Default: 10.
        body (list[str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PaginatedListOfRecordDto]
     """


    kwargs = _get_kwargs(
        body=body,
mini_id=mini_id,
session_id=session_id,
page_number=page_number,
page_size=page_size,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    body: list[str],
    mini_id: str,
    session_id: Union[None, Unset, str] = UNSET,
    page_number: Union[Unset, int] = 1,
    page_size: Union[Unset, int] = 10,

) -> Optional[PaginatedListOfRecordDto]:
    """ 
    Args:
        mini_id (str):
        session_id (Union[None, Unset, str]):
        page_number (Union[Unset, int]):  Default: 1.
        page_size (Union[Unset, int]):  Default: 10.
        body (list[str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PaginatedListOfRecordDto
     """


    return (await asyncio_detailed(
        client=client,
body=body,
mini_id=mini_id,
session_id=session_id,
page_number=page_number,
page_size=page_size,

    )).parsed



from typing import Literal, cast

RecordState = Literal['active', 'archived', 'deleted', 'draft', 'processing']

RECORD_STATE_VALUES: set[RecordState] = { 'active', 'archived', 'deleted', 'draft', 'processing',  }

def check_record_state(value: str) -> RecordState:
    if value in RECORD_STATE_VALUES:
        return cast(RecordState, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {RECORD_STATE_VALUES!r}")



from collections.abc import Mapping
from typing import Any, TypeVar, Optional, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast
from typing import Union






T = TypeVar("T", bound="RecordAuthorizationDto")



@_attrs_define
class RecordAuthorizationDto:
    """ 
        Attributes:
            users (Union[Unset, list[str]]):
            minis (Union[Unset, list[str]]):
            organizations (Union[Unset, list[str]]):
            sessions (Union[Unset, list[str]]):
     """

    users: Union[Unset, list[str]] = UNSET
    minis: Union[Unset, list[str]] = UNSET
    organizations: Union[Unset, list[str]] = UNSET
    sessions: Union[Unset, list[str]] = UNSET





    def to_dict(self) -> dict[str, Any]:
        users: Union[Unset, list[str]] = UNSET
        if not isinstance(self.users, Unset):
            users = self.users



        minis: Union[Unset, list[str]] = UNSET
        if not isinstance(self.minis, Unset):
            minis = self.minis



        organizations: Union[Unset, list[str]] = UNSET
        if not isinstance(self.organizations, Unset):
            organizations = self.organizations



        sessions: Union[Unset, list[str]] = UNSET
        if not isinstance(self.sessions, Unset):
            sessions = self.sessions




        field_dict: dict[str, Any] = {}

        field_dict.update({
        })
        if users is not UNSET:
            field_dict["users"] = users
        if minis is not UNSET:
            field_dict["minis"] = minis
        if organizations is not UNSET:
            field_dict["organizations"] = organizations
        if sessions is not UNSET:
            field_dict["sessions"] = sessions

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        users = cast(list[str], d.pop("users", UNSET))


        minis = cast(list[str], d.pop("minis", UNSET))


        organizations = cast(list[str], d.pop("organizations", UNSET))


        sessions = cast(list[str], d.pop("sessions", UNSET))


        record_authorization_dto = cls(
            users=users,
            minis=minis,
            organizations=organizations,
            sessions=sessions,
        )

        return record_authorization_dto




from collections.abc import Mapping
from typing import Any, TypeVar, Optional, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast
from typing import Union

if TYPE_CHECKING:
  from ..models.upsert_record_dto import UpsertRecordDto





T = TypeVar("T", bound="FailedUpsertItem")



@_attrs_define
class FailedUpsertItem:
    """ 
        Attributes:
            item (Union[Unset, UpsertRecordDto]):
            error (Union[Unset, str]):
     """

    item: Union[Unset, 'UpsertRecordDto'] = UNSET
    error: Union[Unset, str] = UNSET





    def to_dict(self) -> dict[str, Any]:
        from ..models.upsert_record_dto import UpsertRecordDto
        item: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.item, Unset):
            item = self.item.to_dict()

        error = self.error


        field_dict: dict[str, Any] = {}

        field_dict.update({
        })
        if item is not UNSET:
            field_dict["item"] = item
        if error is not UNSET:
            field_dict["error"] = error

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.upsert_record_dto import UpsertRecordDto
        d = dict(src_dict)
        _item = d.pop("item", UNSET)
        item: Union[Unset, UpsertRecordDto]
        if isinstance(_item,  Unset):
            item = UNSET
        else:
            item = UpsertRecordDto.from_dict(_item)




        error = d.pop("error", UNSET)

        failed_upsert_item = cls(
            item=item,
            error=error,
        )

        return failed_upsert_item




""" Contains all the data models used in inputs/outputs """

from .add_attachments_body import AddAttachmentsBody
from .batch_upsert_result import BatchUpsertResult
from .create_group_command import CreateGroupCommand
from .create_mini_command import CreateMiniCommand
from .create_record_command import CreateRecordCommand
from .create_record_command_tags import CreateRecordCommandTags
from .cursor_paginated_list_of_record_dto import CursorPaginatedListOfRecordDto
from .document_file_dto import DocumentFileDto
from .document_file_metadata_dto import DocumentFileMetadataDto
from .failed_upsert_item import FailedUpsertItem
from .form_field import FormField
from .form_field_dto import FormFieldDto
from .form_field_type import FormFieldType
from .group_dto import GroupDto
from .http_validation_problem_details import HttpValidationProblemDetails
from .http_validation_problem_details_errors import HttpValidationProblemDetailsErrors
from .mini_dto import MiniDto
from .mini_template_dto import MiniTemplateDto
from .paginated_list_of_record_dto import PaginatedListOfRecordDto
from .patch_mini_command import PatchMiniCommand
from .problem_details import ProblemDetails
from .record_attachment_dto import RecordAttachmentDto
from .record_attachment_dto_metadata_type_0 import RecordAttachmentDtoMetadataType0
from .record_authorization_dto import RecordAuthorizationDto
from .record_dto import RecordDto
from .record_dto_tags import RecordDtoTags
from .record_relation_dto import RecordRelationDto
from .record_state import RecordState
from .record_tag_dto import RecordTagDto
from .slim_mini_dto import SlimMiniDto
from .tool_dto import ToolDto
from .update_attachments_body import UpdateAttachmentsBody
from .update_group_command import UpdateGroupCommand
from .update_mini_command import UpdateMiniCommand
from .update_mini_template_workspaces_command import UpdateMiniTemplateWorkspacesCommand
from .update_record_command import UpdateRecordCommand
from .update_record_command_tags import UpdateRecordCommandTags
from .upsert_record_dto import UpsertRecordDto
from .upsert_record_dto_tags import UpsertRecordDtoTags
from .upsert_records_by_external_uri_command import UpsertRecordsByExternalUriCommand
from .user_dto import UserDto
from .user_to_mini_dto import UserToMiniDto
from .workspace_dto import WorkspaceDto

__all__ = (
    "AddAttachmentsBody",
    "BatchUpsertResult",
    "CreateGroupCommand",
    "CreateMiniCommand",
    "CreateRecordCommand",
    "CreateRecordCommandTags",
    "CursorPaginatedListOfRecordDto",
    "DocumentFileDto",
    "DocumentFileMetadataDto",
    "FailedUpsertItem",
    "FormField",
    "FormFieldDto",
    "FormFieldType",
    "GroupDto",
    "HttpValidationProblemDetails",
    "HttpValidationProblemDetailsErrors",
    "MiniDto",
    "MiniTemplateDto",
    "PaginatedListOfRecordDto",
    "PatchMiniCommand",
    "ProblemDetails",
    "RecordAttachmentDto",
    "RecordAttachmentDtoMetadataType0",
    "RecordAuthorizationDto",
    "RecordDto",
    "RecordDtoTags",
    "RecordRelationDto",
    "RecordState",
    "RecordTagDto",
    "SlimMiniDto",
    "ToolDto",
    "UpdateAttachmentsBody",
    "UpdateGroupCommand",
    "UpdateMiniCommand",
    "UpdateMiniTemplateWorkspacesCommand",
    "UpdateRecordCommand",
    "UpdateRecordCommandTags",
    "UpsertRecordDto",
    "UpsertRecordDtoTags",
    "UpsertRecordsByExternalUriCommand",
    "UserDto",
    "UserToMiniDto",
    "WorkspaceDto",
)



from collections.abc import Mapping
from typing import Any, TypeVar, Optional, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast, Union
from typing import Union






T = TypeVar("T", bound="CreateMiniCommand")



@_attrs_define
class CreateMiniCommand:
    """ 
        Attributes:
            name (Union[Unset, str]):
            description (Union[None, Unset, str]):
            template_id (Union[None, Unset, str]):
            external_id (Union[None, Unset, str]):
            external_source (Union[None, Unset, str]):
     """

    name: Union[Unset, str] = UNSET
    description: Union[None, Unset, str] = UNSET
    template_id: Union[None, Unset, str] = UNSET
    external_id: Union[None, Unset, str] = UNSET
    external_source: Union[None, Unset, str] = UNSET





    def to_dict(self) -> dict[str, Any]:
        name = self.name

        description: Union[None, Unset, str]
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        template_id: Union[None, Unset, str]
        if isinstance(self.template_id, Unset):
            template_id = UNSET
        else:
            template_id = self.template_id

        external_id: Union[None, Unset, str]
        if isinstance(self.external_id, Unset):
            external_id = UNSET
        else:
            external_id = self.external_id

        external_source: Union[None, Unset, str]
        if isinstance(self.external_source, Unset):
            external_source = UNSET
        else:
            external_source = self.external_source


        field_dict: dict[str, Any] = {}

        field_dict.update({
        })
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if template_id is not UNSET:
            field_dict["templateId"] = template_id
        if external_id is not UNSET:
            field_dict["externalId"] = external_id
        if external_source is not UNSET:
            field_dict["externalSource"] = external_source

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name", UNSET)

        def _parse_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        description = _parse_description(d.pop("description", UNSET))


        def _parse_template_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        template_id = _parse_template_id(d.pop("templateId", UNSET))


        def _parse_external_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        external_id = _parse_external_id(d.pop("externalId", UNSET))


        def _parse_external_source(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        external_source = _parse_external_source(d.pop("externalSource", UNSET))


        create_mini_command = cls(
            name=name,
            description=description,
            template_id=template_id,
            external_id=external_id,
            external_source=external_source,
        )

        return create_mini_command




from collections.abc import Mapping
from typing import Any, TypeVar, Optional, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.record_state import check_record_state
from ..models.record_state import RecordState
from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
from typing import cast, Union
from typing import Union
import datetime

if TYPE_CHECKING:
  from ..models.record_authorization_dto import RecordAuthorizationDto
  from ..models.record_attachment_dto import RecordAttachmentDto
  from ..models.record_relation_dto import RecordRelationDto
  from ..models.record_dto_tags import RecordDtoTags





T = TypeVar("T", bound="RecordDto")



@_attrs_define
class RecordDto:
    """ 
        Attributes:
            id (Union[Unset, str]):
            title (Union[Unset, str]):
            description (Union[None, Unset, str]):
            created_at (Union[Unset, datetime.datetime]):
            updated_at (Union[Unset, datetime.datetime]):
            event_date (Union[None, Unset, datetime.datetime]):
            created_by (Union[Unset, str]):
            updated_by (Union[None, Unset, str]):
            state (Union[Unset, RecordState]):
            ttl (Union[None, Unset, int]):
            schema (Union[Unset, Any]):
            content (Union[Unset, Any]):
            attachments (Union[Unset, list['RecordAttachmentDto']]):
            authorization (Union[Unset, RecordAuthorizationDto]):
            relations (Union[Unset, list['RecordRelationDto']]):
            external_uri (Union[None, Unset, str]):
            labels (Union[Unset, list[str]]):
            tags (Union[Unset, RecordDtoTags]):
     """

    id: Union[Unset, str] = UNSET
    title: Union[Unset, str] = UNSET
    description: Union[None, Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    event_date: Union[None, Unset, datetime.datetime] = UNSET
    created_by: Union[Unset, str] = UNSET
    updated_by: Union[None, Unset, str] = UNSET
    state: Union[Unset, RecordState] = UNSET
    ttl: Union[None, Unset, int] = UNSET
    schema: Union[Unset, Any] = UNSET
    content: Union[Unset, Any] = UNSET
    attachments: Union[Unset, list['RecordAttachmentDto']] = UNSET
    authorization: Union[Unset, 'RecordAuthorizationDto'] = UNSET
    relations: Union[Unset, list['RecordRelationDto']] = UNSET
    external_uri: Union[None, Unset, str] = UNSET
    labels: Union[Unset, list[str]] = UNSET
    tags: Union[Unset, 'RecordDtoTags'] = UNSET





    def to_dict(self) -> dict[str, Any]:
        from ..models.record_authorization_dto import RecordAuthorizationDto
        from ..models.record_attachment_dto import RecordAttachmentDto
        from ..models.record_relation_dto import RecordRelationDto
        from ..models.record_dto_tags import RecordDtoTags
        id = self.id

        title = self.title

        description: Union[None, Unset, str]
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        event_date: Union[None, Unset, str]
        if isinstance(self.event_date, Unset):
            event_date = UNSET
        elif isinstance(self.event_date, datetime.datetime):
            event_date = self.event_date.isoformat()
        else:
            event_date = self.event_date

        created_by = self.created_by

        updated_by: Union[None, Unset, str]
        if isinstance(self.updated_by, Unset):
            updated_by = UNSET
        else:
            updated_by = self.updated_by

        state: Union[Unset, str] = UNSET
        if not isinstance(self.state, Unset):
            state = self.state


        ttl: Union[None, Unset, int]
        if isinstance(self.ttl, Unset):
            ttl = UNSET
        else:
            ttl = self.ttl

        schema = self.schema

        content = self.content

        attachments: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.attachments, Unset):
            attachments = []
            for attachments_item_data in self.attachments:
                attachments_item = attachments_item_data.to_dict()
                attachments.append(attachments_item)



        authorization: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.authorization, Unset):
            authorization = self.authorization.to_dict()

        relations: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.relations, Unset):
            relations = []
            for relations_item_data in self.relations:
                relations_item = relations_item_data.to_dict()
                relations.append(relations_item)



        external_uri: Union[None, Unset, str]
        if isinstance(self.external_uri, Unset):
            external_uri = UNSET
        else:
            external_uri = self.external_uri

        labels: Union[Unset, list[str]] = UNSET
        if not isinstance(self.labels, Unset):
            labels = self.labels



        tags: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags.to_dict()


        field_dict: dict[str, Any] = {}

        field_dict.update({
        })
        if id is not UNSET:
            field_dict["id"] = id
        if title is not UNSET:
            field_dict["title"] = title
        if description is not UNSET:
            field_dict["description"] = description
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at
        if event_date is not UNSET:
            field_dict["eventDate"] = event_date
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by
        if updated_by is not UNSET:
            field_dict["updatedBy"] = updated_by
        if state is not UNSET:
            field_dict["state"] = state
        if ttl is not UNSET:
            field_dict["ttl"] = ttl
        if schema is not UNSET:
            field_dict["schema"] = schema
        if content is not UNSET:
            field_dict["content"] = content
        if attachments is not UNSET:
            field_dict["attachments"] = attachments
        if authorization is not UNSET:
            field_dict["authorization"] = authorization
        if relations is not UNSET:
            field_dict["relations"] = relations
        if external_uri is not UNSET:
            field_dict["externalUri"] = external_uri
        if labels is not UNSET:
            field_dict["labels"] = labels
        if tags is not UNSET:
            field_dict["tags"] = tags

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.record_authorization_dto import RecordAuthorizationDto
        from ..models.record_attachment_dto import RecordAttachmentDto
        from ..models.record_relation_dto import RecordRelationDto
        from ..models.record_dto_tags import RecordDtoTags
        d = dict(src_dict)
        id = d.pop("id", UNSET)

        title = d.pop("title", UNSET)

        def _parse_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        description = _parse_description(d.pop("description", UNSET))


        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at,  Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)




        _updated_at = d.pop("updatedAt", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at,  Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)




        def _parse_event_date(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                event_date_type_0 = isoparse(data)



                return event_date_type_0
            except: # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        event_date = _parse_event_date(d.pop("eventDate", UNSET))


        created_by = d.pop("createdBy", UNSET)

        def _parse_updated_by(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        updated_by = _parse_updated_by(d.pop("updatedBy", UNSET))


        _state = d.pop("state", UNSET)
        state: Union[Unset, RecordState]
        if isinstance(_state,  Unset):
            state = UNSET
        else:
            state = check_record_state(_state)




        def _parse_ttl(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        ttl = _parse_ttl(d.pop("ttl", UNSET))


        schema = d.pop("schema", UNSET)

        content = d.pop("content", UNSET)

        attachments = []
        _attachments = d.pop("attachments", UNSET)
        for attachments_item_data in (_attachments or []):
            attachments_item = RecordAttachmentDto.from_dict(attachments_item_data)



            attachments.append(attachments_item)


        _authorization = d.pop("authorization", UNSET)
        authorization: Union[Unset, RecordAuthorizationDto]
        if isinstance(_authorization,  Unset):
            authorization = UNSET
        else:
            authorization = RecordAuthorizationDto.from_dict(_authorization)




        relations = []
        _relations = d.pop("relations", UNSET)
        for relations_item_data in (_relations or []):
            relations_item = RecordRelationDto.from_dict(relations_item_data)



            relations.append(relations_item)


        def _parse_external_uri(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        external_uri = _parse_external_uri(d.pop("externalUri", UNSET))


        labels = cast(list[str], d.pop("labels", UNSET))


        _tags = d.pop("tags", UNSET)
        tags: Union[Unset, RecordDtoTags]
        if isinstance(_tags,  Unset):
            tags = UNSET
        else:
            tags = RecordDtoTags.from_dict(_tags)




        record_dto = cls(
            id=id,
            title=title,
            description=description,
            created_at=created_at,
            updated_at=updated_at,
            event_date=event_date,
            created_by=created_by,
            updated_by=updated_by,
            state=state,
            ttl=ttl,
            schema=schema,
            content=content,
            attachments=attachments,
            authorization=authorization,
            relations=relations,
            external_uri=external_uri,
            labels=labels,
            tags=tags,
        )

        return record_dto




from collections.abc import Mapping
from typing import Any, TypeVar, Optional, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
from typing import cast, Union
from typing import Union
import datetime

if TYPE_CHECKING:
  from ..models.record_attachment_dto_metadata_type_0 import RecordAttachmentDtoMetadataType0





T = TypeVar("T", bound="RecordAttachmentDto")



@_attrs_define
class RecordAttachmentDto:
    """ 
        Attributes:
            id (Union[Unset, str]):
            file_name (Union[Unset, str]):
            content_type (Union[Unset, str]):
            size (Union[Unset, int]):
            checksum (Union[None, Unset, str]):
            uri (Union[Unset, str]):
            created_at (Union[None, Unset, datetime.datetime]):
            updated_at (Union[None, Unset, datetime.datetime]):
            created_by (Union[None, Unset, str]):
            updated_by (Union[None, Unset, str]):
            metadata (Union['RecordAttachmentDtoMetadataType0', None, Unset]):
     """

    id: Union[Unset, str] = UNSET
    file_name: Union[Unset, str] = UNSET
    content_type: Union[Unset, str] = UNSET
    size: Union[Unset, int] = UNSET
    checksum: Union[None, Unset, str] = UNSET
    uri: Union[Unset, str] = UNSET
    created_at: Union[None, Unset, datetime.datetime] = UNSET
    updated_at: Union[None, Unset, datetime.datetime] = UNSET
    created_by: Union[None, Unset, str] = UNSET
    updated_by: Union[None, Unset, str] = UNSET
    metadata: Union['RecordAttachmentDtoMetadataType0', None, Unset] = UNSET





    def to_dict(self) -> dict[str, Any]:
        from ..models.record_attachment_dto_metadata_type_0 import RecordAttachmentDtoMetadataType0
        id = self.id

        file_name = self.file_name

        content_type = self.content_type

        size = self.size

        checksum: Union[None, Unset, str]
        if isinstance(self.checksum, Unset):
            checksum = UNSET
        else:
            checksum = self.checksum

        uri = self.uri

        created_at: Union[None, Unset, str]
        if isinstance(self.created_at, Unset):
            created_at = UNSET
        elif isinstance(self.created_at, datetime.datetime):
            created_at = self.created_at.isoformat()
        else:
            created_at = self.created_at

        updated_at: Union[None, Unset, str]
        if isinstance(self.updated_at, Unset):
            updated_at = UNSET
        elif isinstance(self.updated_at, datetime.datetime):
            updated_at = self.updated_at.isoformat()
        else:
            updated_at = self.updated_at

        created_by: Union[None, Unset, str]
        if isinstance(self.created_by, Unset):
            created_by = UNSET
        else:
            created_by = self.created_by

        updated_by: Union[None, Unset, str]
        if isinstance(self.updated_by, Unset):
            updated_by = UNSET
        else:
            updated_by = self.updated_by

        metadata: Union[None, Unset, dict[str, Any]]
        if isinstance(self.metadata, Unset):
            metadata = UNSET
        elif isinstance(self.metadata, RecordAttachmentDtoMetadataType0):
            metadata = self.metadata.to_dict()
        else:
            metadata = self.metadata


        field_dict: dict[str, Any] = {}

        field_dict.update({
        })
        if id is not UNSET:
            field_dict["id"] = id
        if file_name is not UNSET:
            field_dict["fileName"] = file_name
        if content_type is not UNSET:
            field_dict["contentType"] = content_type
        if size is not UNSET:
            field_dict["size"] = size
        if checksum is not UNSET:
            field_dict["checksum"] = checksum
        if uri is not UNSET:
            field_dict["uri"] = uri
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by
        if updated_by is not UNSET:
            field_dict["updatedBy"] = updated_by
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.record_attachment_dto_metadata_type_0 import RecordAttachmentDtoMetadataType0
        d = dict(src_dict)
        id = d.pop("id", UNSET)

        file_name = d.pop("fileName", UNSET)

        content_type = d.pop("contentType", UNSET)

        size = d.pop("size", UNSET)

        def _parse_checksum(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        checksum = _parse_checksum(d.pop("checksum", UNSET))


        uri = d.pop("uri", UNSET)

        def _parse_created_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                created_at_type_0 = isoparse(data)



                return created_at_type_0
            except: # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        created_at = _parse_created_at(d.pop("createdAt", UNSET))


        def _parse_updated_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                updated_at_type_0 = isoparse(data)



                return updated_at_type_0
            except: # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        updated_at = _parse_updated_at(d.pop("updatedAt", UNSET))


        def _parse_created_by(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        created_by = _parse_created_by(d.pop("createdBy", UNSET))


        def _parse_updated_by(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        updated_by = _parse_updated_by(d.pop("updatedBy", UNSET))


        def _parse_metadata(data: object) -> Union['RecordAttachmentDtoMetadataType0', None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_type_0 = RecordAttachmentDtoMetadataType0.from_dict(data)



                return metadata_type_0
            except: # noqa: E722
                pass
            return cast(Union['RecordAttachmentDtoMetadataType0', None, Unset], data)

        metadata = _parse_metadata(d.pop("metadata", UNSET))


        record_attachment_dto = cls(
            id=id,
            file_name=file_name,
            content_type=content_type,
            size=size,
            checksum=checksum,
            uri=uri,
            created_at=created_at,
            updated_at=updated_at,
            created_by=created_by,
            updated_by=updated_by,
            metadata=metadata,
        )

        return record_attachment_dto




from collections.abc import Mapping
from typing import Any, TypeVar, Optional, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.record_state import check_record_state
from ..models.record_state import RecordState
from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
from typing import cast, Union
from typing import Union
import datetime

if TYPE_CHECKING:
  from ..models.record_relation_dto import RecordRelationDto
  from ..models.upsert_record_dto_tags import UpsertRecordDtoTags
  from ..models.record_authorization_dto import RecordAuthorizationDto





T = TypeVar("T", bound="UpsertRecordDto")



@_attrs_define
class UpsertRecordDto:
    """ 
        Attributes:
            external_uri (Union[Unset, str]):
            title (Union[Unset, str]):
            description (Union[None, Unset, str]):
            mini_id (Union[Unset, str]):
            event_date (Union[None, Unset, datetime.datetime]):
            schema (Union[Unset, Any]):
            content (Union[Unset, Any]):
            relations (Union[Unset, list['RecordRelationDto']]):
            labels (Union[Unset, list[str]]):
            tags (Union[Unset, UpsertRecordDtoTags]):
            authorization (Union['RecordAuthorizationDto', None, Unset]):
            state (Union[None, RecordState, Unset]):
     """

    external_uri: Union[Unset, str] = UNSET
    title: Union[Unset, str] = UNSET
    description: Union[None, Unset, str] = UNSET
    mini_id: Union[Unset, str] = UNSET
    event_date: Union[None, Unset, datetime.datetime] = UNSET
    schema: Union[Unset, Any] = UNSET
    content: Union[Unset, Any] = UNSET
    relations: Union[Unset, list['RecordRelationDto']] = UNSET
    labels: Union[Unset, list[str]] = UNSET
    tags: Union[Unset, 'UpsertRecordDtoTags'] = UNSET
    authorization: Union['RecordAuthorizationDto', None, Unset] = UNSET
    state: Union[None, RecordState, Unset] = UNSET





    def to_dict(self) -> dict[str, Any]:
        from ..models.record_relation_dto import RecordRelationDto
        from ..models.upsert_record_dto_tags import UpsertRecordDtoTags
        from ..models.record_authorization_dto import RecordAuthorizationDto
        external_uri = self.external_uri

        title = self.title

        description: Union[None, Unset, str]
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        mini_id = self.mini_id

        event_date: Union[None, Unset, str]
        if isinstance(self.event_date, Unset):
            event_date = UNSET
        elif isinstance(self.event_date, datetime.datetime):
            event_date = self.event_date.isoformat()
        else:
            event_date = self.event_date

        schema = self.schema

        content = self.content

        relations: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.relations, Unset):
            relations = []
            for relations_item_data in self.relations:
                relations_item = relations_item_data.to_dict()
                relations.append(relations_item)



        labels: Union[Unset, list[str]] = UNSET
        if not isinstance(self.labels, Unset):
            labels = self.labels



        tags: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags.to_dict()

        authorization: Union[None, Unset, dict[str, Any]]
        if isinstance(self.authorization, Unset):
            authorization = UNSET
        elif isinstance(self.authorization, RecordAuthorizationDto):
            authorization = self.authorization.to_dict()
        else:
            authorization = self.authorization

        state: Union[None, Unset, str]
        if isinstance(self.state, Unset):
            state = UNSET
        elif isinstance(self.state, str):
            state = self.state
        else:
            state = self.state


        field_dict: dict[str, Any] = {}

        field_dict.update({
        })
        if external_uri is not UNSET:
            field_dict["externalUri"] = external_uri
        if title is not UNSET:
            field_dict["title"] = title
        if description is not UNSET:
            field_dict["description"] = description
        if mini_id is not UNSET:
            field_dict["miniId"] = mini_id
        if event_date is not UNSET:
            field_dict["eventDate"] = event_date
        if schema is not UNSET:
            field_dict["schema"] = schema
        if content is not UNSET:
            field_dict["content"] = content
        if relations is not UNSET:
            field_dict["relations"] = relations
        if labels is not UNSET:
            field_dict["labels"] = labels
        if tags is not UNSET:
            field_dict["tags"] = tags
        if authorization is not UNSET:
            field_dict["authorization"] = authorization
        if state is not UNSET:
            field_dict["state"] = state

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.record_relation_dto import RecordRelationDto
        from ..models.upsert_record_dto_tags import UpsertRecordDtoTags
        from ..models.record_authorization_dto import RecordAuthorizationDto
        d = dict(src_dict)
        external_uri = d.pop("externalUri", UNSET)

        title = d.pop("title", UNSET)

        def _parse_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        description = _parse_description(d.pop("description", UNSET))


        mini_id = d.pop("miniId", UNSET)

        def _parse_event_date(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                event_date_type_0 = isoparse(data)



                return event_date_type_0
            except: # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        event_date = _parse_event_date(d.pop("eventDate", UNSET))


        schema = d.pop("schema", UNSET)

        content = d.pop("content", UNSET)

        relations = []
        _relations = d.pop("relations", UNSET)
        for relations_item_data in (_relations or []):
            relations_item = RecordRelationDto.from_dict(relations_item_data)



            relations.append(relations_item)


        labels = cast(list[str], d.pop("labels", UNSET))


        _tags = d.pop("tags", UNSET)
        tags: Union[Unset, UpsertRecordDtoTags]
        if isinstance(_tags,  Unset):
            tags = UNSET
        else:
            tags = UpsertRecordDtoTags.from_dict(_tags)




        def _parse_authorization(data: object) -> Union['RecordAuthorizationDto', None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                authorization_type_0 = RecordAuthorizationDto.from_dict(data)



                return authorization_type_0
            except: # noqa: E722
                pass
            return cast(Union['RecordAuthorizationDto', None, Unset], data)

        authorization = _parse_authorization(d.pop("authorization", UNSET))


        def _parse_state(data: object) -> Union[None, RecordState, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                state_type_0 = check_record_state(data)



                return state_type_0
            except: # noqa: E722
                pass
            return cast(Union[None, RecordState, Unset], data)

        state = _parse_state(d.pop("state", UNSET))


        upsert_record_dto = cls(
            external_uri=external_uri,
            title=title,
            description=description,
            mini_id=mini_id,
            event_date=event_date,
            schema=schema,
            content=content,
            relations=relations,
            labels=labels,
            tags=tags,
            authorization=authorization,
            state=state,
        )

        return upsert_record_dto




from http import HTTPStatus
from typing import Any, Optional, Union, cast

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.user_dto import UserDto
from typing import cast



def _get_kwargs(
    
) -> dict[str, Any]:
    

    

    

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/api/Users",
    }


    return _kwargs



def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[list['UserDto']]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in (_response_200):
            response_200_item = UserDto.from_dict(response_200_item_data)



            response_200.append(response_200_item)

        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[list['UserDto']]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],

) -> Response[list['UserDto']]:
    """ 
    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[list['UserDto']]
     """


    kwargs = _get_kwargs(
        
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    *,
    client: Union[AuthenticatedClient, Client],

) -> Optional[list['UserDto']]:
    """ 
    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        list['UserDto']
     """


    return sync_detailed(
        client=client,

    ).parsed

async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],

) -> Response[list['UserDto']]:
    """ 
    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[list['UserDto']]
     """


    kwargs = _get_kwargs(
        
    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],

) -> Optional[list['UserDto']]:
    """ 
    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        list['UserDto']
     """


    return (await asyncio_detailed(
        client=client,

    )).parsed



from http import HTTPStatus
from typing import Any, Optional, Union, cast

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...types import UNSET, Unset
from typing import cast
from typing import cast, Union
from typing import Union



def _get_kwargs(
    *,
    mini_id: str,
    session_id: Union[None, Unset, str] = UNSET,

) -> dict[str, Any]:
    

    

    params: dict[str, Any] = {}

    params["miniId"] = mini_id

    json_session_id: Union[None, Unset, str]
    if isinstance(session_id, Unset):
        json_session_id = UNSET
    else:
        json_session_id = session_id
    params["sessionId"] = json_session_id


    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}


    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/api/Records/filters/labels",
        "params": params,
    }


    return _kwargs



def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[list[str]]:
    if response.status_code == 200:
        response_200 = cast(list[str], response.json())

        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[list[str]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    mini_id: str,
    session_id: Union[None, Unset, str] = UNSET,

) -> Response[list[str]]:
    """ 
    Args:
        mini_id (str):
        session_id (Union[None, Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[list[str]]
     """


    kwargs = _get_kwargs(
        mini_id=mini_id,
session_id=session_id,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    mini_id: str,
    session_id: Union[None, Unset, str] = UNSET,

) -> Optional[list[str]]:
    """ 
    Args:
        mini_id (str):
        session_id (Union[None, Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        list[str]
     """


    return sync_detailed(
        client=client,
mini_id=mini_id,
session_id=session_id,

    ).parsed

async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    mini_id: str,
    session_id: Union[None, Unset, str] = UNSET,

) -> Response[list[str]]:
    """ 
    Args:
        mini_id (str):
        session_id (Union[None, Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[list[str]]
     """


    kwargs = _get_kwargs(
        mini_id=mini_id,
session_id=session_id,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    mini_id: str,
    session_id: Union[None, Unset, str] = UNSET,

) -> Optional[list[str]]:
    """ 
    Args:
        mini_id (str):
        session_id (Union[None, Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        list[str]
     """


    return (await asyncio_detailed(
        client=client,
mini_id=mini_id,
session_id=session_id,

    )).parsed



from collections.abc import Mapping
from typing import Any, TypeVar, Optional, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast
from typing import Union

if TYPE_CHECKING:
  from ..models.upsert_record_dto import UpsertRecordDto





T = TypeVar("T", bound="UpsertRecordsByExternalUriCommand")



@_attrs_define
class UpsertRecordsByExternalUriCommand:
    """ 
        Attributes:
            items (Union[Unset, list['UpsertRecordDto']]):
     """

    items: Union[Unset, list['UpsertRecordDto']] = UNSET





    def to_dict(self) -> dict[str, Any]:
        from ..models.upsert_record_dto import UpsertRecordDto
        items: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.items, Unset):
            items = []
            for items_item_data in self.items:
                items_item = items_item_data.to_dict()
                items.append(items_item)




        field_dict: dict[str, Any] = {}

        field_dict.update({
        })
        if items is not UNSET:
            field_dict["items"] = items

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.upsert_record_dto import UpsertRecordDto
        d = dict(src_dict)
        items = []
        _items = d.pop("items", UNSET)
        for items_item_data in (_items or []):
            items_item = UpsertRecordDto.from_dict(items_item_data)



            items.append(items_item)


        upsert_records_by_external_uri_command = cls(
            items=items,
        )

        return upsert_records_by_external_uri_command




from http import HTTPStatus
from typing import Any, Optional, Union, cast

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from typing import cast



def _get_kwargs(
    record_id: str,
    *,
    body: list[str],

) -> dict[str, Any]:
    headers: dict[str, Any] = {}


    

    

    _kwargs: dict[str, Any] = {
        "method": "delete",
        "url": "/api/Records/{record_id}/attachments".format(record_id=record_id,),
    }

    _kwargs["json"] = body




    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs



def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Any]:
    if response.status_code == 204:
        return None

    if response.status_code == 404:
        return None

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    record_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: list[str],

) -> Response[Any]:
    """ 
    Args:
        record_id (str):
        body (list[str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
     """


    kwargs = _get_kwargs(
        record_id=record_id,
body=body,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    record_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: list[str],

) -> Response[Any]:
    """ 
    Args:
        record_id (str):
        body (list[str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
     """


    kwargs = _get_kwargs(
        record_id=record_id,
body=body,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)




from http import HTTPStatus
from typing import Any, Optional, Union, cast

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.cursor_paginated_list_of_record_dto import CursorPaginatedListOfRecordDto
from ...models.record_state import check_record_state
from ...models.record_state import RecordState
from ...types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
from typing import cast, Union
from typing import Union
import datetime



def _get_kwargs(
    *,
    mini_id: str,
    record_ids: Union[None, Unset, list[str]] = UNSET,
    session_id: Union[None, Unset, str] = UNSET,
    record_id: Union[None, Unset, str] = UNSET,
    before_cursor: Union[None, Unset, str] = UNSET,
    after_cursor: Union[None, Unset, str] = UNSET,
    limit_before: Union[None, Unset, int] = UNSET,
    limit_after: Union[None, Unset, int] = UNSET,
    labels: Union[None, Unset, list[str]] = UNSET,
    states: Union[None, Unset, list[RecordState]] = UNSET,
    start_date: Union[None, Unset, datetime.datetime] = UNSET,
    end_date: Union[None, Unset, datetime.datetime] = UNSET,
    created_by: Union[None, Unset, list[str]] = UNSET,
    updated_by: Union[None, Unset, list[str]] = UNSET,
    sort_descending: Union[Unset, bool] = True,

) -> dict[str, Any]:
    

    

    params: dict[str, Any] = {}

    params["miniId"] = mini_id

    json_record_ids: Union[None, Unset, list[str]]
    if isinstance(record_ids, Unset):
        json_record_ids = UNSET
    elif isinstance(record_ids, list):
        json_record_ids = record_ids


    else:
        json_record_ids = record_ids
    params["recordIds"] = json_record_ids

    json_session_id: Union[None, Unset, str]
    if isinstance(session_id, Unset):
        json_session_id = UNSET
    else:
        json_session_id = session_id
    params["sessionId"] = json_session_id

    json_record_id: Union[None, Unset, str]
    if isinstance(record_id, Unset):
        json_record_id = UNSET
    else:
        json_record_id = record_id
    params["recordId"] = json_record_id

    json_before_cursor: Union[None, Unset, str]
    if isinstance(before_cursor, Unset):
        json_before_cursor = UNSET
    else:
        json_before_cursor = before_cursor
    params["beforeCursor"] = json_before_cursor

    json_after_cursor: Union[None, Unset, str]
    if isinstance(after_cursor, Unset):
        json_after_cursor = UNSET
    else:
        json_after_cursor = after_cursor
    params["afterCursor"] = json_after_cursor

    json_limit_before: Union[None, Unset, int]
    if isinstance(limit_before, Unset):
        json_limit_before = UNSET
    else:
        json_limit_before = limit_before
    params["limitBefore"] = json_limit_before

    json_limit_after: Union[None, Unset, int]
    if isinstance(limit_after, Unset):
        json_limit_after = UNSET
    else:
        json_limit_after = limit_after
    params["limitAfter"] = json_limit_after

    json_labels: Union[None, Unset, list[str]]
    if isinstance(labels, Unset):
        json_labels = UNSET
    elif isinstance(labels, list):
        json_labels = labels


    else:
        json_labels = labels
    params["labels"] = json_labels

    json_states: Union[None, Unset, list[str]]
    if isinstance(states, Unset):
        json_states = UNSET
    elif isinstance(states, list):
        json_states = []
        for states_type_0_item_data in states:
            states_type_0_item: str = states_type_0_item_data
            json_states.append(states_type_0_item)


    else:
        json_states = states
    params["states"] = json_states

    json_start_date: Union[None, Unset, str]
    if isinstance(start_date, Unset):
        json_start_date = UNSET
    elif isinstance(start_date, datetime.datetime):
        json_start_date = start_date.isoformat()
    else:
        json_start_date = start_date
    params["startDate"] = json_start_date

    json_end_date: Union[None, Unset, str]
    if isinstance(end_date, Unset):
        json_end_date = UNSET
    elif isinstance(end_date, datetime.datetime):
        json_end_date = end_date.isoformat()
    else:
        json_end_date = end_date
    params["endDate"] = json_end_date

    json_created_by: Union[None, Unset, list[str]]
    if isinstance(created_by, Unset):
        json_created_by = UNSET
    elif isinstance(created_by, list):
        json_created_by = created_by


    else:
        json_created_by = created_by
    params["createdBy"] = json_created_by

    json_updated_by: Union[None, Unset, list[str]]
    if isinstance(updated_by, Unset):
        json_updated_by = UNSET
    elif isinstance(updated_by, list):
        json_updated_by = updated_by


    else:
        json_updated_by = updated_by
    params["updatedBy"] = json_updated_by

    params["sortDescending"] = sort_descending


    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}


    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/api/Records",
        "params": params,
    }


    return _kwargs



def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[CursorPaginatedListOfRecordDto]:
    if response.status_code == 200:
        response_200 = CursorPaginatedListOfRecordDto.from_dict(response.json())



        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[CursorPaginatedListOfRecordDto]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    mini_id: str,
    record_ids: Union[None, Unset, list[str]] = UNSET,
    session_id: Union[None, Unset, str] = UNSET,
    record_id: Union[None, Unset, str] = UNSET,
    before_cursor: Union[None, Unset, str] = UNSET,
    after_cursor: Union[None, Unset, str] = UNSET,
    limit_before: Union[None, Unset, int] = UNSET,
    limit_after: Union[None, Unset, int] = UNSET,
    labels: Union[None, Unset, list[str]] = UNSET,
    states: Union[None, Unset, list[RecordState]] = UNSET,
    start_date: Union[None, Unset, datetime.datetime] = UNSET,
    end_date: Union[None, Unset, datetime.datetime] = UNSET,
    created_by: Union[None, Unset, list[str]] = UNSET,
    updated_by: Union[None, Unset, list[str]] = UNSET,
    sort_descending: Union[Unset, bool] = True,

) -> Response[CursorPaginatedListOfRecordDto]:
    """ 
    Args:
        mini_id (str):
        record_ids (Union[None, Unset, list[str]]):
        session_id (Union[None, Unset, str]):
        record_id (Union[None, Unset, str]):
        before_cursor (Union[None, Unset, str]):
        after_cursor (Union[None, Unset, str]):
        limit_before (Union[None, Unset, int]):
        limit_after (Union[None, Unset, int]):
        labels (Union[None, Unset, list[str]]):
        states (Union[None, Unset, list[RecordState]]):
        start_date (Union[None, Unset, datetime.datetime]):
        end_date (Union[None, Unset, datetime.datetime]):
        created_by (Union[None, Unset, list[str]]):
        updated_by (Union[None, Unset, list[str]]):
        sort_descending (Union[Unset, bool]):  Default: True.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[CursorPaginatedListOfRecordDto]
     """


    kwargs = _get_kwargs(
        mini_id=mini_id,
record_ids=record_ids,
session_id=session_id,
record_id=record_id,
before_cursor=before_cursor,
after_cursor=after_cursor,
limit_before=limit_before,
limit_after=limit_after,
labels=labels,
states=states,
start_date=start_date,
end_date=end_date,
created_by=created_by,
updated_by=updated_by,
sort_descending=sort_descending,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    mini_id: str,
    record_ids: Union[None, Unset, list[str]] = UNSET,
    session_id: Union[None, Unset, str] = UNSET,
    record_id: Union[None, Unset, str] = UNSET,
    before_cursor: Union[None, Unset, str] = UNSET,
    after_cursor: Union[None, Unset, str] = UNSET,
    limit_before: Union[None, Unset, int] = UNSET,
    limit_after: Union[None, Unset, int] = UNSET,
    labels: Union[None, Unset, list[str]] = UNSET,
    states: Union[None, Unset, list[RecordState]] = UNSET,
    start_date: Union[None, Unset, datetime.datetime] = UNSET,
    end_date: Union[None, Unset, datetime.datetime] = UNSET,
    created_by: Union[None, Unset, list[str]] = UNSET,
    updated_by: Union[None, Unset, list[str]] = UNSET,
    sort_descending: Union[Unset, bool] = True,

) -> Optional[CursorPaginatedListOfRecordDto]:
    """ 
    Args:
        mini_id (str):
        record_ids (Union[None, Unset, list[str]]):
        session_id (Union[None, Unset, str]):
        record_id (Union[None, Unset, str]):
        before_cursor (Union[None, Unset, str]):
        after_cursor (Union[None, Unset, str]):
        limit_before (Union[None, Unset, int]):
        limit_after (Union[None, Unset, int]):
        labels (Union[None, Unset, list[str]]):
        states (Union[None, Unset, list[RecordState]]):
        start_date (Union[None, Unset, datetime.datetime]):
        end_date (Union[None, Unset, datetime.datetime]):
        created_by (Union[None, Unset, list[str]]):
        updated_by (Union[None, Unset, list[str]]):
        sort_descending (Union[Unset, bool]):  Default: True.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        CursorPaginatedListOfRecordDto
     """


    return sync_detailed(
        client=client,
mini_id=mini_id,
record_ids=record_ids,
session_id=session_id,
record_id=record_id,
before_cursor=before_cursor,
after_cursor=after_cursor,
limit_before=limit_before,
limit_after=limit_after,
labels=labels,
states=states,
start_date=start_date,
end_date=end_date,
created_by=created_by,
updated_by=updated_by,
sort_descending=sort_descending,

    ).parsed

async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    mini_id: str,
    record_ids: Union[None, Unset, list[str]] = UNSET,
    session_id: Union[None, Unset, str] = UNSET,
    record_id: Union[None, Unset, str] = UNSET,
    before_cursor: Union[None, Unset, str] = UNSET,
    after_cursor: Union[None, Unset, str] = UNSET,
    limit_before: Union[None, Unset, int] = UNSET,
    limit_after: Union[None, Unset, int] = UNSET,
    labels: Union[None, Unset, list[str]] = UNSET,
    states: Union[None, Unset, list[RecordState]] = UNSET,
    start_date: Union[None, Unset, datetime.datetime] = UNSET,
    end_date: Union[None, Unset, datetime.datetime] = UNSET,
    created_by: Union[None, Unset, list[str]] = UNSET,
    updated_by: Union[None, Unset, list[str]] = UNSET,
    sort_descending: Union[Unset, bool] = True,

) -> Response[CursorPaginatedListOfRecordDto]:
    """ 
    Args:
        mini_id (str):
        record_ids (Union[None, Unset, list[str]]):
        session_id (Union[None, Unset, str]):
        record_id (Union[None, Unset, str]):
        before_cursor (Union[None, Unset, str]):
        after_cursor (Union[None, Unset, str]):
        limit_before (Union[None, Unset, int]):
        limit_after (Union[None, Unset, int]):
        labels (Union[None, Unset, list[str]]):
        states (Union[None, Unset, list[RecordState]]):
        start_date (Union[None, Unset, datetime.datetime]):
        end_date (Union[None, Unset, datetime.datetime]):
        created_by (Union[None, Unset, list[str]]):
        updated_by (Union[None, Unset, list[str]]):
        sort_descending (Union[Unset, bool]):  Default: True.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[CursorPaginatedListOfRecordDto]
     """


    kwargs = _get_kwargs(
        mini_id=mini_id,
record_ids=record_ids,
session_id=session_id,
record_id=record_id,
before_cursor=before_cursor,
after_cursor=after_cursor,
limit_before=limit_before,
limit_after=limit_after,
labels=labels,
states=states,
start_date=start_date,
end_date=end_date,
created_by=created_by,
updated_by=updated_by,
sort_descending=sort_descending,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    mini_id: str,
    record_ids: Union[None, Unset, list[str]] = UNSET,
    session_id: Union[None, Unset, str] = UNSET,
    record_id: Union[None, Unset, str] = UNSET,
    before_cursor: Union[None, Unset, str] = UNSET,
    after_cursor: Union[None, Unset, str] = UNSET,
    limit_before: Union[None, Unset, int] = UNSET,
    limit_after: Union[None, Unset, int] = UNSET,
    labels: Union[None, Unset, list[str]] = UNSET,
    states: Union[None, Unset, list[RecordState]] = UNSET,
    start_date: Union[None, Unset, datetime.datetime] = UNSET,
    end_date: Union[None, Unset, datetime.datetime] = UNSET,
    created_by: Union[None, Unset, list[str]] = UNSET,
    updated_by: Union[None, Unset, list[str]] = UNSET,
    sort_descending: Union[Unset, bool] = True,

) -> Optional[CursorPaginatedListOfRecordDto]:
    """ 
    Args:
        mini_id (str):
        record_ids (Union[None, Unset, list[str]]):
        session_id (Union[None, Unset, str]):
        record_id (Union[None, Unset, str]):
        before_cursor (Union[None, Unset, str]):
        after_cursor (Union[None, Unset, str]):
        limit_before (Union[None, Unset, int]):
        limit_after (Union[None, Unset, int]):
        labels (Union[None, Unset, list[str]]):
        states (Union[None, Unset, list[RecordState]]):
        start_date (Union[None, Unset, datetime.datetime]):
        end_date (Union[None, Unset, datetime.datetime]):
        created_by (Union[None, Unset, list[str]]):
        updated_by (Union[None, Unset, list[str]]):
        sort_descending (Union[Unset, bool]):  Default: True.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        CursorPaginatedListOfRecordDto
     """


    return (await asyncio_detailed(
        client=client,
mini_id=mini_id,
record_ids=record_ids,
session_id=session_id,
record_id=record_id,
before_cursor=before_cursor,
after_cursor=after_cursor,
limit_before=limit_before,
limit_after=limit_after,
labels=labels,
states=states,
start_date=start_date,
end_date=end_date,
created_by=created_by,
updated_by=updated_by,
sort_descending=sort_descending,

    )).parsed



from collections.abc import Mapping
from typing import Any, TypeVar, Optional, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast
from typing import Union






T = TypeVar("T", bound="UpdateMiniTemplateWorkspacesCommand")



@_attrs_define
class UpdateMiniTemplateWorkspacesCommand:
    """ 
        Attributes:
            mini_template_id (Union[Unset, str]):
            workspace_ids (Union[Unset, list[str]]):
     """

    mini_template_id: Union[Unset, str] = UNSET
    workspace_ids: Union[Unset, list[str]] = UNSET





    def to_dict(self) -> dict[str, Any]:
        mini_template_id = self.mini_template_id

        workspace_ids: Union[Unset, list[str]] = UNSET
        if not isinstance(self.workspace_ids, Unset):
            workspace_ids = self.workspace_ids




        field_dict: dict[str, Any] = {}

        field_dict.update({
        })
        if mini_template_id is not UNSET:
            field_dict["miniTemplateId"] = mini_template_id
        if workspace_ids is not UNSET:
            field_dict["workspaceIds"] = workspace_ids

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        mini_template_id = d.pop("miniTemplateId", UNSET)

        workspace_ids = cast(list[str], d.pop("workspaceIds", UNSET))


        update_mini_template_workspaces_command = cls(
            mini_template_id=mini_template_id,
            workspace_ids=workspace_ids,
        )

        return update_mini_template_workspaces_command




from collections.abc import Mapping
from typing import Any, TypeVar, Optional, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast
from typing import cast, Union
from typing import Union

if TYPE_CHECKING:
  from ..models.workspace_dto import WorkspaceDto
  from ..models.mini_template_dto import MiniTemplateDto
  from ..models.document_file_dto import DocumentFileDto





T = TypeVar("T", bound="MiniDto")



@_attrs_define
class MiniDto:
    """ 
        Attributes:
            id (Union[Unset, int]):
            name (Union[Unset, str]):
            description (Union[None, Unset, str]):
            instructions (Union[None, Unset, str]):
            template_id (Union[None, Unset, str]):
            template (Union['MiniTemplateDto', None, Unset]):
            workspaces (Union[Unset, list['WorkspaceDto']]):
            document_files (Union[Unset, list['DocumentFileDto']]):
     """

    id: Union[Unset, int] = UNSET
    name: Union[Unset, str] = UNSET
    description: Union[None, Unset, str] = UNSET
    instructions: Union[None, Unset, str] = UNSET
    template_id: Union[None, Unset, str] = UNSET
    template: Union['MiniTemplateDto', None, Unset] = UNSET
    workspaces: Union[Unset, list['WorkspaceDto']] = UNSET
    document_files: Union[Unset, list['DocumentFileDto']] = UNSET





    def to_dict(self) -> dict[str, Any]:
        from ..models.workspace_dto import WorkspaceDto
        from ..models.mini_template_dto import MiniTemplateDto
        from ..models.document_file_dto import DocumentFileDto
        id = self.id

        name = self.name

        description: Union[None, Unset, str]
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        instructions: Union[None, Unset, str]
        if isinstance(self.instructions, Unset):
            instructions = UNSET
        else:
            instructions = self.instructions

        template_id: Union[None, Unset, str]
        if isinstance(self.template_id, Unset):
            template_id = UNSET
        else:
            template_id = self.template_id

        template: Union[None, Unset, dict[str, Any]]
        if isinstance(self.template, Unset):
            template = UNSET
        elif isinstance(self.template, MiniTemplateDto):
            template = self.template.to_dict()
        else:
            template = self.template

        workspaces: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.workspaces, Unset):
            workspaces = []
            for workspaces_item_data in self.workspaces:
                workspaces_item = workspaces_item_data.to_dict()
                workspaces.append(workspaces_item)



        document_files: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.document_files, Unset):
            document_files = []
            for document_files_item_data in self.document_files:
                document_files_item = document_files_item_data.to_dict()
                document_files.append(document_files_item)




        field_dict: dict[str, Any] = {}

        field_dict.update({
        })
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if instructions is not UNSET:
            field_dict["instructions"] = instructions
        if template_id is not UNSET:
            field_dict["templateId"] = template_id
        if template is not UNSET:
            field_dict["template"] = template
        if workspaces is not UNSET:
            field_dict["workspaces"] = workspaces
        if document_files is not UNSET:
            field_dict["documentFiles"] = document_files

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.workspace_dto import WorkspaceDto
        from ..models.mini_template_dto import MiniTemplateDto
        from ..models.document_file_dto import DocumentFileDto
        d = dict(src_dict)
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        def _parse_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        description = _parse_description(d.pop("description", UNSET))


        def _parse_instructions(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        instructions = _parse_instructions(d.pop("instructions", UNSET))


        def _parse_template_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        template_id = _parse_template_id(d.pop("templateId", UNSET))


        def _parse_template(data: object) -> Union['MiniTemplateDto', None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                template_type_0 = MiniTemplateDto.from_dict(data)



                return template_type_0
            except: # noqa: E722
                pass
            return cast(Union['MiniTemplateDto', None, Unset], data)

        template = _parse_template(d.pop("template", UNSET))


        workspaces = []
        _workspaces = d.pop("workspaces", UNSET)
        for workspaces_item_data in (_workspaces or []):
            workspaces_item = WorkspaceDto.from_dict(workspaces_item_data)



            workspaces.append(workspaces_item)


        document_files = []
        _document_files = d.pop("documentFiles", UNSET)
        for document_files_item_data in (_document_files or []):
            document_files_item = DocumentFileDto.from_dict(document_files_item_data)



            document_files.append(document_files_item)


        mini_dto = cls(
            id=id,
            name=name,
            description=description,
            instructions=instructions,
            template_id=template_id,
            template=template,
            workspaces=workspaces,
            document_files=document_files,
        )

        return mini_dto




from collections.abc import Mapping
from typing import Any, TypeVar, Optional, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import Union






T = TypeVar("T", bound="RecordTagDto")



@_attrs_define
class RecordTagDto:
    """ 
        Attributes:
            key (Union[Unset, str]):
            value (Union[Unset, Any]):
     """

    key: Union[Unset, str] = UNSET
    value: Union[Unset, Any] = UNSET





    def to_dict(self) -> dict[str, Any]:
        key = self.key

        value = self.value


        field_dict: dict[str, Any] = {}

        field_dict.update({
        })
        if key is not UNSET:
            field_dict["key"] = key
        if value is not UNSET:
            field_dict["value"] = value

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        key = d.pop("key", UNSET)

        value = d.pop("value", UNSET)

        record_tag_dto = cls(
            key=key,
            value=value,
        )

        return record_tag_dto




from collections.abc import Mapping
from typing import Any, TypeVar, Optional, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast
from typing import Union

if TYPE_CHECKING:
  from ..models.failed_upsert_item import FailedUpsertItem





T = TypeVar("T", bound="BatchUpsertResult")



@_attrs_define
class BatchUpsertResult:
    """ 
        Attributes:
            successful_ids (Union[Unset, list[str]]):
            failed (Union[Unset, list['FailedUpsertItem']]):
            success_count (Union[Unset, int]):
            failure_count (Union[Unset, int]):
     """

    successful_ids: Union[Unset, list[str]] = UNSET
    failed: Union[Unset, list['FailedUpsertItem']] = UNSET
    success_count: Union[Unset, int] = UNSET
    failure_count: Union[Unset, int] = UNSET





    def to_dict(self) -> dict[str, Any]:
        from ..models.failed_upsert_item import FailedUpsertItem
        successful_ids: Union[Unset, list[str]] = UNSET
        if not isinstance(self.successful_ids, Unset):
            successful_ids = self.successful_ids



        failed: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.failed, Unset):
            failed = []
            for failed_item_data in self.failed:
                failed_item = failed_item_data.to_dict()
                failed.append(failed_item)



        success_count = self.success_count

        failure_count = self.failure_count


        field_dict: dict[str, Any] = {}

        field_dict.update({
        })
        if successful_ids is not UNSET:
            field_dict["successfulIds"] = successful_ids
        if failed is not UNSET:
            field_dict["failed"] = failed
        if success_count is not UNSET:
            field_dict["successCount"] = success_count
        if failure_count is not UNSET:
            field_dict["failureCount"] = failure_count

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.failed_upsert_item import FailedUpsertItem
        d = dict(src_dict)
        successful_ids = cast(list[str], d.pop("successfulIds", UNSET))


        failed = []
        _failed = d.pop("failed", UNSET)
        for failed_item_data in (_failed or []):
            failed_item = FailedUpsertItem.from_dict(failed_item_data)



            failed.append(failed_item)


        success_count = d.pop("successCount", UNSET)

        failure_count = d.pop("failureCount", UNSET)

        batch_upsert_result = cls(
            successful_ids=successful_ids,
            failed=failed,
            success_count=success_count,
            failure_count=failure_count,
        )

        return batch_upsert_result




from typing import Literal, cast

FormFieldType = Literal['checkbox', 'dateTime', 'radio', 'text']

FORM_FIELD_TYPE_VALUES: set[FormFieldType] = { 'checkbox', 'dateTime', 'radio', 'text',  }

def check_form_field_type(value: str) -> FormFieldType:
    if value in FORM_FIELD_TYPE_VALUES:
        return cast(FormFieldType, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {FORM_FIELD_TYPE_VALUES!r}")



from collections.abc import Mapping
from typing import Any, TypeVar, Optional, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast, Union
from typing import Union






T = TypeVar("T", bound="ProblemDetails")



@_attrs_define
class ProblemDetails:
    """ 
        Attributes:
            type_ (Union[None, Unset, str]):
            title (Union[None, Unset, str]):
            status (Union[None, Unset, int]):
            detail (Union[None, Unset, str]):
            instance (Union[None, Unset, str]):
     """

    type_: Union[None, Unset, str] = UNSET
    title: Union[None, Unset, str] = UNSET
    status: Union[None, Unset, int] = UNSET
    detail: Union[None, Unset, str] = UNSET
    instance: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        type_: Union[None, Unset, str]
        if isinstance(self.type_, Unset):
            type_ = UNSET
        else:
            type_ = self.type_

        title: Union[None, Unset, str]
        if isinstance(self.title, Unset):
            title = UNSET
        else:
            title = self.title

        status: Union[None, Unset, int]
        if isinstance(self.status, Unset):
            status = UNSET
        else:
            status = self.status

        detail: Union[None, Unset, str]
        if isinstance(self.detail, Unset):
            detail = UNSET
        else:
            detail = self.detail

        instance: Union[None, Unset, str]
        if isinstance(self.instance, Unset):
            instance = UNSET
        else:
            instance = self.instance


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if type_ is not UNSET:
            field_dict["type"] = type_
        if title is not UNSET:
            field_dict["title"] = title
        if status is not UNSET:
            field_dict["status"] = status
        if detail is not UNSET:
            field_dict["detail"] = detail
        if instance is not UNSET:
            field_dict["instance"] = instance

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        def _parse_type_(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        type_ = _parse_type_(d.pop("type", UNSET))


        def _parse_title(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        title = _parse_title(d.pop("title", UNSET))


        def _parse_status(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        status = _parse_status(d.pop("status", UNSET))


        def _parse_detail(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        detail = _parse_detail(d.pop("detail", UNSET))


        def _parse_instance(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        instance = _parse_instance(d.pop("instance", UNSET))


        problem_details = cls(
            type_=type_,
            title=title,
            status=status,
            detail=detail,
            instance=instance,
        )


        problem_details.additional_properties = d
        return problem_details

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, Optional, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast
from typing import cast, Union
from typing import Union

if TYPE_CHECKING:
  from ..models.record_dto import RecordDto





T = TypeVar("T", bound="CursorPaginatedListOfRecordDto")



@_attrs_define
class CursorPaginatedListOfRecordDto:
    """ 
        Attributes:
            items (Union[Unset, list['RecordDto']]):
            before_cursor (Union[None, Unset, str]):
            after_cursor (Union[None, Unset, str]):
     """

    items: Union[Unset, list['RecordDto']] = UNSET
    before_cursor: Union[None, Unset, str] = UNSET
    after_cursor: Union[None, Unset, str] = UNSET





    def to_dict(self) -> dict[str, Any]:
        from ..models.record_dto import RecordDto
        items: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.items, Unset):
            items = []
            for items_item_data in self.items:
                items_item = items_item_data.to_dict()
                items.append(items_item)



        before_cursor: Union[None, Unset, str]
        if isinstance(self.before_cursor, Unset):
            before_cursor = UNSET
        else:
            before_cursor = self.before_cursor

        after_cursor: Union[None, Unset, str]
        if isinstance(self.after_cursor, Unset):
            after_cursor = UNSET
        else:
            after_cursor = self.after_cursor


        field_dict: dict[str, Any] = {}

        field_dict.update({
        })
        if items is not UNSET:
            field_dict["items"] = items
        if before_cursor is not UNSET:
            field_dict["beforeCursor"] = before_cursor
        if after_cursor is not UNSET:
            field_dict["afterCursor"] = after_cursor

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.record_dto import RecordDto
        d = dict(src_dict)
        items = []
        _items = d.pop("items", UNSET)
        for items_item_data in (_items or []):
            items_item = RecordDto.from_dict(items_item_data)



            items.append(items_item)


        def _parse_before_cursor(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        before_cursor = _parse_before_cursor(d.pop("beforeCursor", UNSET))


        def _parse_after_cursor(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        after_cursor = _parse_after_cursor(d.pop("afterCursor", UNSET))


        cursor_paginated_list_of_record_dto = cls(
            items=items,
            before_cursor=before_cursor,
            after_cursor=after_cursor,
        )

        return cursor_paginated_list_of_record_dto




from collections.abc import Mapping
from typing import Any, TypeVar, Optional, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
from typing import cast, Union
from typing import Union
import datetime






T = TypeVar("T", bound="FormField")



@_attrs_define
class FormField:
    """ 
        Attributes:
            key (Union[Unset, str]):
            value_string (Union[None, Unset, str]):
            value_number (Union[None, Unset, float]):
            value_date (Union[None, Unset, datetime.datetime]):
     """

    key: Union[Unset, str] = UNSET
    value_string: Union[None, Unset, str] = UNSET
    value_number: Union[None, Unset, float] = UNSET
    value_date: Union[None, Unset, datetime.datetime] = UNSET





    def to_dict(self) -> dict[str, Any]:
        key = self.key

        value_string: Union[None, Unset, str]
        if isinstance(self.value_string, Unset):
            value_string = UNSET
        else:
            value_string = self.value_string

        value_number: Union[None, Unset, float]
        if isinstance(self.value_number, Unset):
            value_number = UNSET
        else:
            value_number = self.value_number

        value_date: Union[None, Unset, str]
        if isinstance(self.value_date, Unset):
            value_date = UNSET
        elif isinstance(self.value_date, datetime.datetime):
            value_date = self.value_date.isoformat()
        else:
            value_date = self.value_date


        field_dict: dict[str, Any] = {}

        field_dict.update({
        })
        if key is not UNSET:
            field_dict["key"] = key
        if value_string is not UNSET:
            field_dict["valueString"] = value_string
        if value_number is not UNSET:
            field_dict["valueNumber"] = value_number
        if value_date is not UNSET:
            field_dict["valueDate"] = value_date

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        key = d.pop("key", UNSET)

        def _parse_value_string(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        value_string = _parse_value_string(d.pop("valueString", UNSET))


        def _parse_value_number(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        value_number = _parse_value_number(d.pop("valueNumber", UNSET))


        def _parse_value_date(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                value_date_type_0 = isoparse(data)



                return value_date_type_0
            except: # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        value_date = _parse_value_date(d.pop("valueDate", UNSET))


        form_field = cls(
            key=key,
            value_string=value_string,
            value_number=value_number,
            value_date=value_date,
        )

        return form_field




from collections.abc import Mapping
from typing import Any, TypeVar, Optional, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast, Union
from typing import Union






T = TypeVar("T", bound="DocumentFileMetadataDto")



@_attrs_define
class DocumentFileMetadataDto:
    """ 
        Attributes:
            content_hash (Union[None, Unset, str]):
     """

    content_hash: Union[None, Unset, str] = UNSET





    def to_dict(self) -> dict[str, Any]:
        content_hash: Union[None, Unset, str]
        if isinstance(self.content_hash, Unset):
            content_hash = UNSET
        else:
            content_hash = self.content_hash


        field_dict: dict[str, Any] = {}

        field_dict.update({
        })
        if content_hash is not UNSET:
            field_dict["contentHash"] = content_hash

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        def _parse_content_hash(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        content_hash = _parse_content_hash(d.pop("contentHash", UNSET))


        document_file_metadata_dto = cls(
            content_hash=content_hash,
        )

        return document_file_metadata_dto




from collections.abc import Mapping
from typing import Any, TypeVar, Optional, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast, Union
from typing import Union






T = TypeVar("T", bound="WorkspaceDto")



@_attrs_define
class WorkspaceDto:
    """ 
        Attributes:
            id (Union[Unset, int]):
            workspace_id (Union[Unset, str]):
            name (Union[Unset, str]):
            organization_id (Union[None, Unset, str]):
     """

    id: Union[Unset, int] = UNSET
    workspace_id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    organization_id: Union[None, Unset, str] = UNSET





    def to_dict(self) -> dict[str, Any]:
        id = self.id

        workspace_id = self.workspace_id

        name = self.name

        organization_id: Union[None, Unset, str]
        if isinstance(self.organization_id, Unset):
            organization_id = UNSET
        else:
            organization_id = self.organization_id


        field_dict: dict[str, Any] = {}

        field_dict.update({
        })
        if id is not UNSET:
            field_dict["id"] = id
        if workspace_id is not UNSET:
            field_dict["workspaceId"] = workspace_id
        if name is not UNSET:
            field_dict["name"] = name
        if organization_id is not UNSET:
            field_dict["organizationId"] = organization_id

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id", UNSET)

        workspace_id = d.pop("workspaceId", UNSET)

        name = d.pop("name", UNSET)

        def _parse_organization_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        organization_id = _parse_organization_id(d.pop("organizationId", UNSET))


        workspace_dto = cls(
            id=id,
            workspace_id=workspace_id,
            name=name,
            organization_id=organization_id,
        )

        return workspace_dto

